@@ -1,4 +1,4 @@
-from typing import Callable, Collection, Optional
+from typing import Callable, Collection
 from unittest import mock
 from unittest.mock import AsyncMock, Mock
 
@@ -72,7 +72,7 @@ class FederationCatchUpTestCases(FederatingHomeserverTestCase):
         return config
 
     async def record_transaction(
-        self, txn: Transaction, json_cb: Optional[Callable[[], JsonDict]]
+        self, txn: Transaction, json_cb: Callable[[], JsonDict] | None
     ) -> JsonDict:
         if json_cb is None:
             # The tests seem to expect that this method raises in this situation.
